@@ -1,33 +1,51 @@
 # -*- coding: utf-8 -*-
+import os.path
+
 import math
 import cairo
+import rsvg
+import gobject
+import gtk.gdk
 
 import glchess.scene
 
-import pieces
-
 from gettext import gettext as _
 
-BACKGROUND_COLOUR    = (0.53, 0.63, 0.75)
-BORDER_COLOUR        = (0.808, 0.361, 0.0)#(0.757, 0.490, 0.067)#(0.36, 0.21, 0.05)
-NUMBERING_COLOUR     = (249.0/255, 172.0/255, 109.0/255)#(249.0/255, 132.0/255, 38.0/255)
-BLACK_SQUARE_COLOURS = {None:                               (0.8, 0.8, 0.8),
-                        glchess.scene.HIGHLIGHT_SELECTED:   (0.3, 1.0, 0.3),
-                        glchess.scene.HIGHLIGHT_CAN_MOVE:   (0.3, 0.3, 0.8),
-                        glchess.scene.HIGHLIGHT_THREATENED: (1.0, 0.8, 0.8),
-                        glchess.scene.HIGHLIGHT_CAN_TAKE:   (1.0, 0.3, 0.3)}
-WHITE_SQUARE_COLOURS = {None:                               (1.0, 1.0, 1.0),
-                        glchess.scene.HIGHLIGHT_SELECTED:   (0.2, 1.0, 0.0),
-                        glchess.scene.HIGHLIGHT_CAN_MOVE:   (0.2, 0.2, 0.8),
-                        glchess.scene.HIGHLIGHT_THREATENED: (1.0, 0.8, 0.8),
-                        glchess.scene.HIGHLIGHT_CAN_TAKE:   (1.0, 0.2, 0.2)}
-PIECE_COLOUR         = (0.0, 0.0, 0.0)
+def parse_colour(colour):
+    assert colour.startswith('#')
+    assert len(colour) == 7
+    r = int(colour[1:3], 16) / 255.
+    g = int(colour[3:5], 16) / 255.
+    b = int(colour[5:7], 16) / 255.
+    return (r, g, b)
+
+def blend_colour(colour_a, colour_b, alpha):
+    a = parse_colour(colour_a)
+    b = parse_colour(colour_b)
+    r = a[0] * alpha + b[0] * (1 - alpha)
+    g = a[1] * alpha + b[1] * (1 - alpha)
+    b = a[2] * alpha + b[2] * (1 - alpha)
+    return (r, g, b)
+
+BORDER_COLOUR        = parse_colour('#2e3436')
+NUMBERING_COLOUR     = parse_colour('#888a85')
+BLACK_SQUARE_COLOURS = {None:                               parse_colour('#babdb6'),
+                        glchess.scene.HIGHLIGHT_SELECTED:   parse_colour('#73d216'),
+                        glchess.scene.HIGHLIGHT_CAN_MOVE:   parse_colour('#3465a4'),
+                        glchess.scene.HIGHLIGHT_THREATENED: blend_colour('#af0000', '#babdb6', 0.2),
+                        glchess.scene.HIGHLIGHT_CAN_TAKE:   blend_colour('#af0000', '#babdb6', 0.8)}
+WHITE_SQUARE_COLOURS = {None:                               parse_colour('#eeeeec'),
+                        glchess.scene.HIGHLIGHT_SELECTED:   parse_colour('#8ae234'),
+                        glchess.scene.HIGHLIGHT_CAN_MOVE:   parse_colour('#204a87'),
+                        glchess.scene.HIGHLIGHT_THREATENED: blend_colour('#cc0000', '#eeeeec', 0.2),
+                        glchess.scene.HIGHLIGHT_CAN_TAKE:   blend_colour('#cc0000', '#eeeeec', 0.8)}
+PIECE_COLOUR         = parse_colour('#000000')
 
 class ChessPiece(glchess.scene.ChessPiece):
     """
     """
 
-    def __init__(self, scene, name, coord, feedback):
+    def __init__(self, scene, name, coord, feedback, style='simple'):
         """
         """
         self.scene = scene
@@ -38,6 +56,8 @@ class ChessPiece(glchess.scene.ChessPiece):
         self.targetPos   = None
         self.moving = False
         self.delete = False # Delete once moved to location
+
+        self.setStyle(style)
         
     def __coordToLocation(self, coord):
         """
@@ -47,6 +67,14 @@ class ChessPiece(glchess.scene.ChessPiece):
         
         return (float(rank), float(file))
 
+    def setStyle(self, style):
+        self.style = style
+        self.path = os.path.join(glchess.defaults.BASE_DIR, 'pieces', style, self.name + '.svg')
+        try:
+            self.svg = rsvg.Handle(file = self.path)
+        except Exception as e:
+            raise Exception('Error reading %s: %s' % (self.path, e))
+
     def move(self, coord, delete, animate = True):
         """Extends glchess.scene.ChessPiece"""
         if not coord:
@@ -127,28 +155,31 @@ class ChessPiece(glchess.scene.ChessPiece):
     def render(self, context):
         """
         """
-        offset = 0
+        offset = self.scene.PIECE_BORDER
         matrix = context.get_matrix()
         x = (self.pos[0] - 4) * self.scene.squareSize
         y = (3 - self.pos[1]) * self.scene.squareSize
+
         context.translate(x, y)
         context.translate(self.scene.squareSize / 2, self.scene.squareSize / 2)
         context.rotate(-self.scene.angle)
-        context.translate(-self.scene.squareSize / 2, -self.scene.squareSize / 2)
 
         # If Face to Face mode is enabled, we rotate the black player's pieces by 180 degrees
         if self.scene.faceToFace and self.name.find('black') != -1:
             context.rotate(math.pi)
-            offset -= self.scene.pieceSize
-        pieces.piece(self.name, context, self.scene.pieceSize, offset, offset)
-        context.fill()
+            offset = - offset - self.scene.pieceSize + self.scene.squareSize
+        
+        context.translate(-self.scene.squareSize / 2 + offset, -self.scene.squareSize / 2 + offset)
+        context.scale(self.scene.pieceSize/self.svg.props.width, self.scene.pieceSize/self.svg.props.height)
+
+        self.svg.render_cairo(context)
         context.set_matrix(matrix)
 
 class Scene(glchess.scene.Scene):
     """
     """    
     BORDER = 6.0
-    PIECE_BORDER = 2.0
+    PIECE_BORDER = 0.0
 
     def __init__(self, feedback):
         """Constructor for a Cairo scene"""
@@ -162,8 +193,13 @@ class Scene(glchess.scene.Scene):
         self.redrawStatic    = True
         self.showNumbering   = False
         self.faceToFace      = False
-    
-    def addChessPiece(self, chessSet, name, coord, feedback):
+        try:
+            pixbuf = gtk.gdk.pixbuf_new_from_file(os.path.join(glchess.defaults.SHARED_IMAGE_DIR, 'baize.png'))
+            (self.background_pixmap, _) = pixbuf.render_pixmap_and_mask()
+        except gobject.GError:
+            self.background_pixmap = None
+
+    def addChessPiece(self, chessSet, name, coord, feedback, style = 'simple'):
         """Add a chess piece model into the scene.
         
         'chessSet' is the name of the chess set (string).
@@ -173,7 +209,7 @@ class Scene(glchess.scene.Scene):
         Returns a reference to this chess piece or raises an exception.
         """
         name = chessSet + name[0].upper() + name[1:]
-        piece = ChessPiece(self, name, coord, feedback)
+        piece = ChessPiece(self, name, coord, feedback, style=style)
         self.pieces.append(piece)
         
         # Redraw the scene
@@ -246,6 +282,12 @@ class Scene(glchess.scene.Scene):
             self.redrawStatic = True
             self.feedback.onRedraw()
 
+    def setPiecesStyle(self, piecesStyle):
+        for piece in self.pieces:
+            piece.setStyle(piecesStyle)
+        self.redrawStatic = True
+        self.feedback.onRedraw()
+    
     def animate(self, timeStep):
         """Extends glchess.scene.Scene"""
         if self.angle == self.targetAngle and len(self._animationQueue) == 0:
@@ -342,15 +384,14 @@ class Scene(glchess.scene.Scene):
         
         context.rotate(self.angle);
 
-    def renderStatic(self, context):
+    def renderStatic(self, context, background_color):
         """Render the static elements in a scene.
         """
         if self.redrawStatic is False:
             return False
         self.redrawStatic = False
 
-        # Clear background
-        context.set_source_rgb(*BACKGROUND_COLOUR)
+        context.set_source_rgb(*background_color)
         context.paint()
         
         # Rotate board
