@@ -346,7 +346,7 @@ games_runtime_init (const char *name)
   }
 #endif /* G_OS_WIN32 */
 
-#if defined(HAVE_GNOME) || defined(HAVE_RSVG_GNOMEVFS) || defined(HAVE_CANBERRA_GTK)
+#if defined(HAVE_GNOME) || defined(HAVE_RSVG_GNOMEVFS) || defined(ENABLE_SOUND)
   /* If we're going to use gconf, gnome-vfs, or canberra, we need to
    * init threads; and this has to be done before calling any other glib functions.
    */
@@ -385,7 +385,7 @@ games_runtime_init (const char *name)
   retval = TRUE;
 #endif /* ENABLE_BINRELOC */
 
-  if (strcmp (app_name, "aisleriot") == 0 || strcmp (app_name, "blackjack") == 0) {
+  if (strcmp (app_name, "aisleriot") == 0) {
     gpl_version = 3;
   } else {
     gpl_version = 2;
